﻿using Hotline.Caching.Interfaces;
using Hotline.CallCenter.Devices;
using Hotline.CallCenter.Tels;
using Hotline.Share.Dtos.TrCallCenter;
using Hotline.Share.Dtos.Users;
using MapsterMapper;
using Tr.Sdk;
using XF.Domain.Cache;
using XF.Domain.Constants;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Users
{
    public class UserDomainService : IUserDomainService, IScopeDependency
    {
        private readonly IRepository<User> _userRepository;
        private readonly IWorkRepository _workRepository;
        private readonly IDeviceManager _deviceManager;
        private readonly IUserCacheManager _userCacheManager;
        private readonly ITelCacheManager _telCacheManager;
        private readonly ITypedCache<Work> _cacheWork;
        private readonly IMapper _mapper;
        private readonly ISystemSettingCacheManager _systemSettingCacheManager;
        private readonly ITrClient _trClient;

        public UserDomainService(
            IRepository<User> userRepository,
            IWorkRepository workRepository,
            IDeviceManager deviceManager,
            ITypedCache<Work> cacheWork,
            IUserCacheManager userCacheManager,
            IMapper mapper, 
            ITelCacheManager telCacheManager,
            ISystemSettingCacheManager systemSettingCacheManager,
            ITrClient trClient)
        {
            _userRepository = userRepository;
            _workRepository = workRepository;
            _deviceManager = deviceManager;
            _userCacheManager = userCacheManager;
            _cacheWork = cacheWork;
            _mapper = mapper;
            _telCacheManager = telCacheManager;
            _systemSettingCacheManager = systemSettingCacheManager;
            _trClient = trClient;
        }

        /// <summary>
        /// 上班
        /// </summary>
        /// <param name="userId">userId</param>
        /// <param name="tel"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task OnDutyAsync(string userId, Tel tel, CancellationToken cancellationToken)
        {
            var user = await _userRepository.GetAsync(userId, cancellationToken);
            if (user is null)
                throw UserFriendlyException.SameMessage("无效的用户编号");

            var workingTel = await _workRepository.GetCurrentWorkByUserAsync(userId, cancellationToken);
            if (workingTel is not null)
                throw UserFriendlyException.SameMessage($"当前用户已上班，userId:{userId}");

            var telIsWorking = await _workRepository.AnyAsync(d => d.TelId == tel.Id && !d.EndTime.HasValue, cancellationToken);
            if (telIsWorking)
                throw UserFriendlyException.SameMessage("当前分机已被占用");

            var work = new Work(userId, user.Name, tel.Id, tel.No);
            await _workRepository.AddAsync(work, cancellationToken);

            if (!string.IsNullOrEmpty(user.StaffNo))
                await _deviceManager.UpdateStaffNoAsync(tel.No, user.StaffNo, tel.LineId, cancellationToken);

            //更新分机组
            foreach (var group in tel.Groups)
            {
                await _deviceManager.AssginConfigGroupAsync(group.No,group.Distribution,new List<string>() {tel.No }, group.Voice, cancellationToken);
            }


        }

        /// <summary>
        /// 下班
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task<WorkDto?> OffDutyAsync(string userId, CancellationToken cancellationToken)
        {
            var work = _userCacheManager.GetWorkByUser(userId);
            var tel = _telCacheManager.GetTel(work.TelNo);
            work.OffDuty();
            await _workRepository.UpdateAsync(work, cancellationToken);
            await _deviceManager.UpdateStaffNoAsync(work.TelNo, string.Empty, tel.LineId, cancellationToken);
            _cacheWork.Remove(work.GetKey(KeyMode.UserId));
            _cacheWork.Remove(work.GetKey(KeyMode.TelNo));
            #region 初始化话机

            //初始化解除静音
            await _deviceManager.UnMuteAsync(tel.No, cancellationToken);

            foreach (var group in tel.Groups)
            {
                await _deviceManager.ModifyGroupExtAsync(group.No, group.Distribution, group.Voice, "", false, cancellationToken);
            }

            #endregion
            return _mapper.Map<WorkDto>(work);
        }


        #region 添添呼

        /// <summary>
        /// 上班
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="TelId"></param>
        /// <param name="TelNo"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task<TrOnDutyResponseDto> TrOnDutyAsync(string userId, string? TelId, string? TelNo, string? TelPwd, string? Description,CancellationToken cancellationToken)
        {
            var workingTel = await _workRepository.GetCurrentWorkByUserAsync(userId, cancellationToken);
            if (workingTel is not null)
            {
                return new TrOnDutyResponseDto() { TelNo = workingTel.TelNo, TelPwd = workingTel.TelPwd, Description = workingTel.Description };
            }

            //检查配置
            bool IsNeedTelNo = bool.Parse(_systemSettingCacheManager.GetSetting(SettingConstants.IsNeedTelNo).SettingValue[0]);
            if (IsNeedTelNo && string.IsNullOrEmpty(TelNo))
            {
                throw UserFriendlyException.SameMessage("当前配置需要选择分机号签入");
            }


            

            var user = await _userRepository.GetAsync(userId, cancellationToken);
            if (user is null)
                throw UserFriendlyException.SameMessage("无效的用户编号");

            bool IsTelNeedVerify = bool.Parse(_systemSettingCacheManager.GetSetting(SettingConstants.IsTelNeedVerify).SettingValue[0]);
            var rsp = new TrOnDutyResponseDto();
            if (!IsNeedTelNo)
            {
                TelNo = user.DefaultTelNo;
                var telModel = await _trClient.QueryTelsAsync(new Tr.Sdk.Tels.QueryTelRequest() { TelNo = TelNo }, cancellationToken);
                TelId = telModel[0].Id;
                rsp.TelPwd = telModel[0].Password;
                rsp.Description = telModel[0].Description;
            }
            

            rsp.TelNo = TelNo;
          

            var telIsWorking = await _workRepository.AnyAsync(d => d.TelId == TelId && !d.EndTime.HasValue, cancellationToken);
            if (telIsWorking)
                throw UserFriendlyException.SameMessage("当前分机已被占用");

            var work = new Work(userId, user.Name, TelId, TelNo,rsp.TelPwd,rsp.Description);
            await _workRepository.AddAsync(work, cancellationToken);

            if (IsTelNeedVerify)
            {
                rsp.TelPwd = string.Empty;
            }
            return rsp;
        }

        /// <summary>
        /// 下班
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task TrOffDutyAsync(string userId,CancellationToken cancellationToken)
        {
            var work = _userCacheManager.GetWorkByUser(userId);
            work.OffDuty();
            await _workRepository.UpdateAsync(work, cancellationToken);
            _cacheWork.Remove(work.GetKey(KeyMode.UserId));
            _cacheWork.Remove(work.GetKey(KeyMode.TelNo));
        }


        #endregion
    }
}
